"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterByPragma = void 0;
const fs_1 = require("fs");
const jest_docblock_1 = require("jest-docblock");
const TEST_BLOCK_REGEX = /(fit|it|xit|test|xtest)(.each\(\S*\))?\(\s*(["'`])(?<testName>[\S ]*)(["'`]),\s*(async\s*)?\((\S*)?\)\s*=>\s*{\s*(?<docBlock>\/\*\*?(.|\r?\n)*?\*\/)/gm;
const filterByPragma = (testFiles, pragmasToMatch) => {
    if (testFiles.length === 0) {
        return [];
    }
    return _filterFileTestCasesByPragma(testFiles, pragmasToMatch);
};
exports.filterByPragma = filterByPragma;
function _filterFileTestCasesByPragma(testFiles, pragmasToMatch) {
    const matchingTestFiles = [];
    for (const testFile of testFiles) {
        const source = fs_1.readFileSync(testFile.path).toString();
        const testBlocks = _findTestBlocksWithPragmas(source);
        if (!testBlocks || testBlocks.length === 0) {
            continue;
        }
        const matchingTestBlocks = testBlocks.filter(testBlock => _doPragmasHaveMatchingValues(testBlock.pragmas, pragmasToMatch));
        if (matchingTestBlocks.length > 0) {
            const testNamePattern = matchingTestBlocks.map(testBlock => `^${_escapeRegExp(testBlock.name)}$`).join('|');
            /** @privateRemarks
             *  Typescript expects JestTest to be readonly.
       *  The workaround for now is to make a clone.
             */
            // TODO: Modify Jest Types to enable modifying this within the Jest-Runner or find a cleaner way to clone this.
            matchingTestFiles.push({ ...testFile, context: {
                    ...testFile.context,
                    config: {
                        ...testFile.context.config,
                        testNamePattern: `(${testNamePattern})`
                    }
                }
            });
        }
    }
    return matchingTestFiles;
}
function _escapeRegExp(string) {
    return string.replace(/[.*+\-?^${}()|[\]\\]/g, '\\$&'); // $& means the whole matched string
}
function _findTestBlocksWithPragmas(source) {
    let match;
    const groups = [];
    do {
        match = TEST_BLOCK_REGEX.exec(source);
        if (match) {
            const group = { ...match.groups };
            const name = group === null || group === void 0 ? void 0 : group.testName;
            const docBlock = group === null || group === void 0 ? void 0 : group.docBlock.replace(/\t/g, '');
            const pragmas = { ...jest_docblock_1.parse(docBlock) };
            if (pragmas !== {}) {
                groups.push({
                    name,
                    docBlock,
                    pragmas
                });
            }
        }
    } while (match);
    return groups;
}
function _doPragmasHaveMatchingValues(pragmas, pragmasToMatch) {
    const pragmaKeysToMatch = Object.keys(pragmasToMatch);
    for (const pragmaKeyToMatch of pragmaKeysToMatch) {
        const pragma = pragmas[pragmaKeyToMatch];
        const pragmaToMatch = pragmasToMatch[pragmaKeyToMatch];
        if (!pragma) {
            return false;
        }
        const hasMatchingPragmaValue = pragmaToMatch.some(valuesToMatch => pragma.includes(valuesToMatch));
        if (hasMatchingPragmaValue) {
            return true;
        }
    }
    return false;
}
//# sourceMappingURL=filter-by-pragma.js.map